import { declare } from '@babel/helper-plugin-utils';
import syntaxPartialApplication from '@babel/plugin-syntax-partial-application';
import { types } from '@babel/core';

var index = declare(api => {
  api.assertVersion("8.0.0-alpha.9");
  function hasArgumentPlaceholder(node) {
    return node.arguments.some(arg => types.isArgumentPlaceholder(arg));
  }
  function unwrapArguments({
    arguments: args
  }, scope) {
    const init = [];
    for (let i = 0; i < args.length; i++) {
      const node = args[i];
      if (!types.isArgumentPlaceholder(node) && !types.isImmutable(node)) {
        const id = scope.generateUidIdentifierBasedOnNode(node, "param");
        scope.push({
          id
        });
        if (types.isSpreadElement(node)) {
          init.push(types.assignmentExpression("=", types.cloneNode(id), types.arrayExpression([types.spreadElement(node.argument)])));
          node.argument = types.cloneNode(id);
        } else {
          init.push(types.assignmentExpression("=", types.cloneNode(id), node));
          args[i] = types.cloneNode(id);
        }
      }
    }
    return init;
  }
  function replacePlaceholders(node, scope) {
    const placeholders = [];
    const newArgs = [];
    node.arguments.forEach(arg => {
      if (types.isArgumentPlaceholder(arg)) {
        const id = scope.generateUid("_argPlaceholder");
        placeholders.push(types.identifier(id));
        newArgs.push(types.identifier(id));
      } else {
        newArgs.push(arg);
      }
    });
    return [placeholders, newArgs];
  }
  return {
    name: "proposal-partial-application",
    inherits: syntaxPartialApplication,
    visitor: {
      CallExpression(path) {
        if (!hasArgumentPlaceholder(path.node)) {
          return;
        }
        const {
          node,
          scope
        } = path;
        const functionLVal = path.scope.generateUidIdentifierBasedOnNode(node.callee);
        const sequenceParts = [];
        const argsInitializers = unwrapArguments(node, scope);
        const [placeholdersParams, args] = replacePlaceholders(node, scope);
        scope.push({
          id: functionLVal
        });
        if (node.callee.type === "MemberExpression") {
          const {
            object: receiver,
            property
          } = node.callee;
          const receiverLVal = path.scope.generateUidIdentifierBasedOnNode(receiver);
          scope.push({
            id: receiverLVal
          });
          sequenceParts.push(types.assignmentExpression("=", types.cloneNode(receiverLVal), receiver), types.assignmentExpression("=", types.cloneNode(functionLVal), types.memberExpression(types.cloneNode(receiverLVal), property)), ...argsInitializers, types.functionExpression(types.isIdentifier(property) ? types.cloneNode(property) : path.scope.generateUidIdentifierBasedOnNode(property), placeholdersParams, types.blockStatement([types.returnStatement(types.callExpression(types.memberExpression(types.cloneNode(functionLVal), types.identifier("call")), [types.cloneNode(receiverLVal), ...args]))], []), false, false));
        } else {
          sequenceParts.push(types.assignmentExpression("=", types.cloneNode(functionLVal), node.callee), ...argsInitializers, types.functionExpression(types.isIdentifier(node.callee) ? types.cloneNode(node.callee) : path.scope.generateUidIdentifierBasedOnNode(node.callee), placeholdersParams, types.blockStatement([types.returnStatement(types.callExpression(types.cloneNode(functionLVal), args))], []), false, false));
        }
        path.replaceWith(types.sequenceExpression(sequenceParts));
      }
    }
  };
});

export { index as default };
//# sourceMappingURL=index.js.map
